@@ -1,4 +1,5 @@
 # Category metadata.
+import sysconfig
 
 # Category icon show in the menu
 ICON = "icons/category.svg"
@@ -8,18 +9,21 @@ ICON = "icons/category.svg"
 BACKGROUND = "light-blue"
 
 
-
 # Location of widget help files.
 WIDGET_HELP_PATH = (
     # Used for development.
     # You still need to build help pages using
-    # make htmlhelp
-    # inside doc folder
-    ("{DEVELOP_ROOT}/doc/_build/htmlhelp/index.html", None),
+    # `make html` inside doc folder
+    ("{DEVELOP_ROOT}/doc/_build/html/index.html", None),
+
+    # Documentation included in wheel
+    # Correct DATA_FILES entry is needed in setup.py and documentation has to be built
+    # before the wheel is created.
+    ("{}/help/orange3-text/index.html".format(sysconfig.get_path("data")), None),
 
     # Online documentation url, used when the local documentation is available.
     # Url should point to a page with a section Widgets. This section should
     # includes links to documentation pages of each widget. Matching is
     # performed by comparing link caption to widget name.
-    ("https://orange3-text.readthedocs.io/en/latest/", "")
-)
\ No newline at end of file
+    ("https://orange3-text.readthedocs.io/en/latest/", ""),
+)
